# coding: UTF-8

'''
検証（画像の生成）

画像を16個生成して、８列・２行で出力する
'''

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os

import tensorflow as tf
import numpy as np

# 使用するモデルを指定
from model import easy_gan as model
# from model import dc_gan1 as model
# from model import dc_gan2 as model
# from model import dc_gan3 as model

FLAGS = tf.app.flags.FLAGS
tf.app.flags.DEFINE_string('output_dir', None,
                           "出力先のディレクトリ")
tf.app.flags.DEFINE_string('train_path', None,
                           "学習結果のファイル")


def _export_images(image):
  image = tf.concat(tf.split(image, 16)[:8], axis=2)
  image = (tf.squeeze(image, axis=0) + 1.0) / 2.0
  image = tf.image.convert_image_dtype(image, tf.uint8)
  encoded_image = tf.image.encode_png(image)

  return encoded_image


def main(args=None):
  assert FLAGS.output_dir, \
    'argument --output_dir is not set.'
  assert os.path.exists(FLAGS.output_dir), \
    '--output_dir path %s is not exist.' % FLAGS.output_dir
  assert FLAGS.train_path, \
    'argument --train_path is not set.'

  inputs = tf.placeholder(tf.float32, shape=[16, model.INPUT_SIZE])
  image = model.generator(inputs, 16)
  iamge = _export_images(image)

  saver = tf.train.Saver(tf.global_variables())

  config = tf.ConfigProto(
    allow_soft_placement=True,
    log_device_placement=False)

  with tf.Session(config=config) as sess:
    saver.restore(sess, FLAGS.train_path)

    noise = np.random.normal(size=model.INPUT_SIZE * 16)
    noise = np.reshape(noise, [16, model.INPUT_SIZE])

    file_name = '%s.png' % (model.NAME)
    output_path = os.path.join(FLAGS.output_dir, file_name)
    images = sess.run(iamge, feed_dict={
      inputs: noise
    })

    with open(output_path, mode='wb') as f:
      f.write(images)


if __name__ == '__main__':
  tf.app.run()
