import tensorflow as tf

'''
モデル DCGAN
'''

NAME = 'dc_gan1'

INPUT_SIZE = 100

def generator(inputs, batch_size, is_train=True):
  with tf.variable_scope('generator'):
    with tf.variable_scope('reshape'):
      fc0 = tf.layers.dense(inputs, 8 * 8 * 256)
      out0 = tf.reshape(fc0, [batch_size, 8, 8, 256])

    with tf.variable_scope('conv_transpose1'):
      out1 = tf.layers.conv2d_transpose(out0, 128, [5, 5], (2, 2),
                                        padding='SAME')
      out1 = tf.layers.batch_normalization(out1, training=is_train)
      out1 = tf.nn.relu(out1)

      print(out1.get_shape())

    with tf.variable_scope('conv_transpose2'):
      out2 = tf.layers.conv2d_transpose(out1, 64, [5, 5], (2, 2),
                                        padding='SAME')
      out2 = tf.layers.batch_normalization(out2, training=is_train)
      out2 = tf.nn.relu(out2)

      print(out2.get_shape())

    with tf.variable_scope('conv_transpose3'):
      out3 = tf.layers.conv2d_transpose(out2, 32, [5, 5], (2, 2),
                                        padding='SAME')
      out3 = tf.layers.batch_normalization(out3, training=is_train)
      out3 = tf.nn.relu(out3)

      print(out3.get_shape())

    with tf.variable_scope('conv_transpose4'):
      out4 = tf.layers.conv2d_transpose(out3, 1, [5, 5], (2, 2),
                                        padding='SAME')
      print(out4.get_shape())

  return tf.nn.tanh(out4)


def discriminator(inputs, batch_size, is_reuse=False, is_train=True):
  with tf.variable_scope('discriminator', reuse=is_reuse):
    with tf.variable_scope('conv1'):
      conv1 = tf.layers.conv2d(inputs, 64, [5, 5], (2, 2),
                               activation=tf.nn.leaky_relu,
                               padding='VALID')

    with tf.variable_scope('conv2'):
      conv2 = tf.layers.conv2d(conv1, 128, [5, 5], (2, 2),
                               activation=tf.nn.leaky_relu,
                               padding='VALID')

    reshaped = tf.reshape(conv2, [batch_size, -1])

    with tf.variable_scope('fc1'):
      fc1 = tf.layers.dense(reshaped, 256,
                            activation=tf.nn.leaky_relu)
      fc1 = tf.layers.dropout(fc1, 0.5, training=is_train)

    with tf.variable_scope('output'):
      output = tf.layers.dense(fc1, 1)
  return output
