import tensorflow as tf

'''
モデル 拡張DCGAN
'''

NAME = 'dc_gan2'

INPUT_SIZE = 100

BLOCK_CHANNELS = [128, 64, 32, 16]


def generator(inputs, batch_size, is_train=True):
  with tf.variable_scope('generator'):
    with tf.variable_scope('reshape'):
      fc1 = tf.layers.dense(inputs, 8 * 8 * 256)
      prev = tf.reshape(fc1, [batch_size, 8, 8, 256])

    for channel in BLOCK_CHANNELS:
      prev = tf.layers.conv2d_transpose(prev, channel, [5, 5], (2, 2),
                                        padding='SAME')
      prev = tf.layers.batch_normalization(prev, training=is_train)
      prev = tf.nn.relu(prev)

    output = tf.layers.conv2d_transpose(prev, 1, [5, 5], (2, 2),
                                        padding='SAME')

  return tf.nn.tanh(output)


def discriminator(inputs, batch_size, is_reuse=False, is_train=True):
  with tf.variable_scope('discriminator', reuse=is_reuse):
    with tf.variable_scope('conv1'):
      conv1 = tf.layers.conv2d(inputs, 64, [5, 5], (2, 2),
                               activation=tf.nn.leaky_relu,
                               padding='VALID')

    with tf.variable_scope('conv2'):
      conv2 = tf.layers.conv2d(conv1, 128, [5, 5], (2, 2),
                               activation=tf.nn.leaky_relu,
                               padding='VALID')

    reshaped = tf.reshape(conv2, [batch_size, -1])

    with tf.variable_scope('fc1'):
      fc1 = tf.layers.dense(reshaped, 256,
                            activation=tf.nn.leaky_relu)
      fc1 = tf.layers.dropout(fc1, 0.5, training=is_train)

    with tf.variable_scope('output'):
      output = tf.layers.dense(fc1, 1)
  return output
