import os

from PIL import Image
from six import unichr

JIS_FILE_PATH = '../data/JIS0208.TXT'
IMAGE_OUTPUT_DIR = '../data/images'

FONT_DIR = '/Library/Fonts/'
EXPORT_POINT = 256

CJK_COMMENT = '# <CJK>'


def _export_glyph_image(code, font_path, font_point, output_dir):
  char = unichr(code)
  font_file_name = os.path.basename(font_path)
  name, ext = os.path.splitext(font_file_name)

  output_dir = os.path.join(output_dir, name)
  output_path = os.path.join(output_dir, '%s-%d.jpg' % (name, code))

  if os.path.exists(output_path):
    return None

  os.makedirs(output_dir, exist_ok=True)

  command = ' '.join(['convert',
                      '-background', 'white',
                      '-fill', 'black',
                      '-font', font_path,
                      '-pointsize', str(font_point),
                      '-quality', str(80),
                      'label:%s' % char,
                      output_path
                      ])
  # print(command)
  os.system(command)

  return output_path


def _resize(output_path, size):
  with Image.open(output_path) as image:
    if image.width == size and image.height == size:
      return

    max_size = max(image.width, image.height)
    new_image = Image.new("L",
                          (max_size, max_size),
                          color=0xffffff)
    new_image.paste(image, ((max_size - image.width) // 2,
                            (max_size - image.height) // 2))
    new_image = new_image.resize((size, size))
    new_image.save(output_path, 'jpeg')


font_paths = os.listdir(FONT_DIR)
font_paths = filter(
  lambda file: file.endswith('.otf'), font_paths)
font_paths = filter(
  lambda file: file.startswith('A-OTF-'), font_paths)
font_paths = list(map(
  lambda file: os.path.join(FONT_DIR, file), font_paths))

with open(JIS_FILE_PATH, 'r') as f:
  for index, line in enumerate(f):
    if line.startswith('#'):
      continue

    if index % 100 == 0:
      print('%d5' % (index))

    token = line.split("\t")
    code = int(token[2], 16)
    comment = token[3][:-1]

    if comment != CJK_COMMENT:
      continue

    for index, font_path in enumerate(font_paths):
      output_path = _export_glyph_image(
        code,
        font_path, EXPORT_POINT,
        IMAGE_OUTPUT_DIR)
      if output_path is not None:
        _resize(output_path, EXPORT_POINT)
